"""
The JARVIS-API is available at https://jarvis.nist.gov/.
Users DO NOT need to create account if they just want to get the data.
Account creation is needed for the users who intendto upload data.
"""
